package conflicts;

import oops.parser.Scanner;
import oops.parser.Parser;
import oops.parser.Set;
import oops.parser.Token;
import java.io.Reader;
import java.io.FilterReader;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.StreamTokenizer;

/** Lexical analyzer for conflicts.ebnf.
    @author Bernd Kuehl, Bernd.Kuehl@informatik.uni-osnabrueck.de
    @author Axel-Tobias Schreiner, axel@informatik.uni-osnabrueck.de
    @author Jan Kraneis, jkraneis@informatik.uni-osnabrueck.de
    @version 1.0
*/

public class Input implements Scanner {
  
  /** A java.io.StramTokenizer.
  */
  protected StreamTokenizer st;
  
  /** An oops.parser.Parser.
  */
  protected Parser parser;

  /** The scan() method needed by the oops.parser.Scanner interface.
      @throws java.io.IOException
  */  
  public void scan (Reader r, Parser parser) throws IOException {
    st = new StreamTokenizer(new BufferedReader(r));

    this.parser = parser;

    advance();				// one token lookahead
  }
  
  /** The atEnd() method needed by the oops.parser.Scanner interface.
      @return true at the end of the input file, else false.
  */
  public boolean atEnd () {
    return st.ttype == st.TT_EOF;
  }

  /** An oops.parser.Set. Should be null or lookahead identifying token.
  */
  protected Set tokenSet;		// null or lookahead identifying token
  
  /** A java.lang.Object.
  */
  protected Object node;		// node representing token

  /** The advance() method needed by the oops.parser.Scanner interface.
      @return true on advance, false at the end of the input file.     
  */
  public boolean advance () throws IOException {
    if (atEnd()) return false;
    switch (st.nextToken()) {
      case st.TT_EOF:	tokenSet = Set.getEOFSet(); return false;
      case st.TT_WORD:	tokenSet = parser.getLitSet(st.sval); break;
      default:		tokenSet = null; break;
    }
    return true;
  }

  /** The tokenSet() method needed by the oops.parser.Scanner interface.
      @return single element lookahead set, null for unidentifiable token.      
  */
  public Set tokenSet () {
    return tokenSet;
  }

  /** The node() method needed by the oops.parser.Scanner interface.
      @return always null.      
  */
  public Object node () {
    return null;
  }

  /** Returns a String representation containing the current position. 
  */
  public String toString () {
    return st==null ? "unconnected" : st.toString();
  }
}
