package ebnf;

import oops.parser.Scanner;
import oops.parser.Parser;
import oops.parser.Set;
import oops.parser.Token;
import java.io.Reader;
import java.io.FilterReader;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.StreamTokenizer;

/** Lexical analyzer for ebnf.ebnf.
    @author Bernd Kuehl, Bernd.Kuehl@informatik.uni-osnabrueck.de
    @author Axel-Tobias Schreiner, axel@informatik.uni-osnabrueck.de
    @author Jan Kraneis, jkraneis@informatik.uni-osnabrueck.de
    @version 1.0
*/

public class Input implements Scanner {

  /** A java.io.StramTokenizer.
  */
  protected StreamTokenizer st;
  
  /** An oops.parser.Parser.
  */
  protected Parser parser;
  
  /** An oops.parser.Set. Stores lookahead for "ID". 
  */
  protected Set ID;
  
  /** An oops.parser.Set. Stores lookahead for "LIT". 
  */
  protected Set LIT;

  /** The scan() method needed by the oops.parser.Scanner interface.
      @throws java.io.IOException
  */
  public void scan (Reader r, Parser parser) throws IOException {
    st = new StreamTokenizer(new BufferedReader(r));
    st.resetSyntax();
    st.slashSlashComments(true); // C and C++ comments
    st.quoteChar('"');           // strings in "
    st.whitespaceChars(0, ' ');  // ignore control-* and space
    st.wordChars('a', 'z');     // identifiers [a-zA-Z_.][a-zA-Z_0-9.]*
    st.wordChars('A', 'Z');
    st.wordChars('_', '_');
    st.wordChars('.', '.');

    this.parser = parser;
    ID = ((Token) parser.getPeer("ID")).getLookahead();
    LIT = ((Token) parser.getPeer("LIT")).getLookahead();

    advance();				// one token lookahead
  }

  /** The atEnd() method needed by the oops.parser.Scanner interface.
      @return true at the end of the input file, else false.
  */
  public boolean atEnd () {
    return st.ttype == st.TT_EOF;
  }

  /** An oops.parser.Set. Should be null or lookahead identifying token.
  */
  protected Set tokenSet;		// null or lookahead identifying token
  
  /** The advance() method needed by the oops.parser.Scanner interface.
      @return true on advance, false at the end of the input file.     
  */
  public boolean advance () throws IOException {
    if (atEnd()) return false;
    switch (st.nextToken()) {
      case st.TT_EOF:  tokenSet = Set.getEOFSet(); return false;
      case st.TT_WORD: tokenSet = ID; break; // id or token
      case '"':        tokenSet = LIT; break;
      default:         tokenSet = parser.getLitSet(""+(char)st.ttype);; break;
    }
    return true;
  }

  /** The tokenSet() method needed by the oops.parser.Scanner interface.
      @return single element lookahead set, null for unidentifiable token.      
  */
  public Set tokenSet () {
    return tokenSet;
  }

  /** The node() method needed by the oops.parser.Scanner interface.
      @return always null.      
  */
  public Object node () {
    return null;
  }

  /** Returns a String representation containing the current position. 
  */
  public String toString () {
    return st==null ? "unconnected" : st.toString();
  }
}
