package scanners;

import oops.parser.Scanner;
import oops.parser.Parser;
import oops.parser.Set;

import java.io.Reader;
import java.io.IOException;

%%

%public
%class JLexScanner

%type boolean
%eofval{
  return false;
%eofval}

%{

  public static class OopsScanner implements Scanner, Action {
    public OopsScanner() {}
    protected Parser parser;
    protected Set NUMBER;
    protected JLexScanner scanner;
    /** initialize, read one symbol ahead.
      @param parser is used to screen symbols.
      */
    public void scan (Reader in, Parser parser) throws IOException {
      this.parser = parser;
      NUMBER = parser.getPeer("NUMBER").getLookahead();
      scanner = new JLexScanner(in);
      scanner.setAction(this);
      advance();
    }
    /** move on to next token.
      @return false if atEnd() becomes true.
      */
    public boolean advance () throws IOException {
      atEnd = !scanner.yylex();
      if(atEnd) {
        tokenSet = Set.getEOFSet();
        node = null;
      }
      return !atEnd;
    }
    protected boolean atEnd;
    /** @return true if positioned beyond tokens.
      */
    public boolean atEnd () {
      return atEnd;
    }
    protected Set tokenSet;
    /** @return single-element lookahead set, null for unidentifiable token.
      */
    public Set tokenSet () {
      return tokenSet;
    }
    protected Object node;
    /** @return node corresponding to token.
      */
    public Object node () {
      return node;
    }
    public void real(String symbol) {
      tokenSet = NUMBER;
      node = new Double(symbol);
      }
    public void integer (String symbol) {
      tokenSet = NUMBER;
      node = new Long(symbol);
      }
    public void character (String symbol) {
      tokenSet = parser.getLitSet(symbol);
      node = null;
      }
  }

  public interface Action {
    public void real(String symbol);
    public void integer(String symbol);
    public void character(String symbol);
  }

  protected Action action;
  protected void setAction(Action action) {
    this.action = action;
  }


%}

comment	= ("#".*)
space	= [\ \t\b\015]+
digit	= [0-9]
integer	= {digit}+
real	= ({digit}+"."{digit}*|{digit}*"."{digit}+)

%%

{space}|{comment}   { }
\n                  { }
{integer}           { action.integer(yytext()); return true; }
{real}              { action.real(yytext()); return true; }
.					{ action.character(yytext()); return true; }
