package scanners;

import oops.parser.Scanner;
import oops.parser.Parser;
import oops.parser.Set;

import java.io.Reader;
import java.io.IOException;

import oolex.Input;
import oolex.Input.EmptyTokenException;
import oolex.Input.IllegalCharacterException;
import oolex.Scan;
import oolex.Scan.Action;
import oolex.Scan.Alt;

import oolex.scanobjects.Char;
import oolex.scanobjects.Int;
import oolex.scanobjects.Flt;
import oolex.scanobjects.SlashSlashComment;
import oolex.scanobjects.HashComment;
import oolex.scanobjects.JavaWhitespace;

/** Lexical analyzer for arithmetic examples.
    @author Bernd Kuehl, Bernd.Kuehl@informatik.uni-osnabrueck.de
    @author Axel-Tobias Schreiner, axel@informatik.uni-osnabrueck.de
    @author Jan Kraneis, jkraneis@informatik.uni-osnabrueck.de
    @version 1.0
*/
public class OOLexScanner implements Scanner {
  /** null or lookahead identifying token */
  protected Set tokenSet;
  /** node representing token */
  protected Object node;
  /** The input instance. */
  protected Input input;
  /** The Set for the category NUMBER */
  protected Set NUMBER;

  /** The scanner. */
  protected final Scan scan = new Alt(new Scan [] {
    new Int().setAction(new Action() {
      public void action(Scan sender, char[] buf, int off, int len) {
        node = sender.genValue(buf, off, len);
        tokenSet = NUMBER;
      }
    }),
    new Flt().setAction(new Action() {
      public void action(Scan sender, char[] buf, int off, int len) {
        node = sender.genValue(buf, off, len);
        tokenSet = NUMBER;
      }
    }),
    new oolex.Scan.Set(";+-*/()", true).setAction(new Action() {
      public void action(Scan sender, char[] buf, int off, int len) {
        node = null;
        tokenSet = parser.getLitSet(""+buf[off]);
      }
    }),
    new SlashSlashComment().setIgnore(true),
    new HashComment().setIgnore(true),
    new JavaWhitespace().setIgnore(true),
    new Char().setAction(new Action() {
      public void action(Scan sender, char[] buf, int off, int len) {
        System.err.println("found unkown character: "+buf[off]);
        new Exception().printStackTrace();
        node = null;
        tokenSet = null; // unkown symbol
      }
    }),
  });
  /** The parser. Used in action methode to set tokenSet. */
  protected Parser parser;


  /** initialize, read one symbol ahead.
    @param parser is used to screen symbols.
    */
  public void scan (Reader in, Parser parser) throws IOException {
    input = new Input(in, 1024, 1024);
    this.parser = parser;
    NUMBER = parser.getPeer("NUMBER").getLookahead();
    advance();
  }
  /** move on to next token.
    @return false if atEnd() becomes true.
    */
  public boolean advance () throws IOException {
    if(atEnd()) return false;
    try {
      boolean result = input.token(scan);
      // EOF reached => set tokenSet for EOF symbol.
      if(!result) {
        node = null;
        tokenSet = Set.getEOFSet();
      }
      return result;
    } catch (EmptyTokenException ete) {
      // can't happen for this scanner...
      System.err.println(ete);
    } catch (IllegalCharacterException ice) {
      // illegal input!
      System.err.println(ice);
    }
    System.exit(1);
    return false; // dummy for compiler
  }
  /** @return true if positioned beyond tokens.
    */
  public boolean atEnd () {
    return input.atEnd();
  }
  /** @return single-element lookahead set, null for unidentifiable token.
    */
  public Set tokenSet () {
    return tokenSet;
  }
  /** @return node corresponding to token.
    */
  public Object node () {
    return node;
  }

  /** @return a String representation. */
  public String toString () {
    return input==null ? "unconnected" : input.toString();
  }
}
