package scanners;

import oops.parser.Scanner;
import oops.parser.Parser;
import oops.parser.Set;
import oops.parser.Token;
import java.io.Reader;
import java.io.FilterReader;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.StreamTokenizer;

/** Lexical analyzer for arithmetic examples.
    @author Bernd Kuehl, Bernd.Kuehl@informatik.uni-osnabrueck.de
    @author Axel-Tobias Schreiner, axel@informatik.uni-osnabrueck.de
    @author Jan Kraneis, jkraneis@informatik.uni-osnabrueck.de
    @version 1.0
*/
public class StreamTokenizerScanner implements Scanner {
  protected StreamTokenizer st;
  protected Parser parser;
  protected Set NUMBER, NL;

  /** The scan() method needed by the oops.parser.Scanner interface.
      @throws java.io.IOException
  */
  public void scan (Reader r, Parser parser) throws IOException {
    st = new StreamTokenizer(new BufferedReader(r));
    st.ordinaryChar('/'); st.ordinaryChar('-'); // would start comment or number
    st.commentChar('#');		// comments from # to end-of-line

    this.parser = parser;
    NUMBER = ((Token)parser.getPeer("NUMBER")).getLookahead();
    NL = parser.getLitSet("\n");

    advance();				// one token lookahead
  }

  /** The atEnd() method needed by the oops.parser.Scanner interface.
      @return true at the end of the input file, else false.
  */
  public boolean atEnd () { return st.ttype == st.TT_EOF; }

  /** An oops.parser.Set. Should be null or lookahead identifying token.
  */
  protected Set tokenSet;		// null or lookahead identifying token
  
  /** A java.lang.Object.
  */
  protected Object node;		// node representing token
  
  /** The advance() method needed by the oops.parser.Scanner interface.
      @return true on advance, false at the end of the input file.     
  */
  public boolean advance () throws IOException {
    if (atEnd()) return false;
    node = null;
    switch (st.nextToken()) {
      case st.TT_EOF:    tokenSet = Set.getEOFSet(); return false;
      case st.TT_EOL:    tokenSet = NL; break;
      case st.TT_NUMBER: node = new Double(st.nval); tokenSet = NUMBER; break;
      case st.TT_WORD:   tokenSet = null; break;
      default:           tokenSet = parser.getLitSet(""+(char)st.ttype); break;
    }
    return true;
  }

  /** The tokenSet() method needed by the oops.parser.Scanner interface.
      @return single element lookahead set, null for unidentifiable token.      
  */
  public Set tokenSet () { 
    return tokenSet;
  }

  /** The node() method needed by the oops.parser.Scanner interface.
      @return associated value for current input symbol.      
  */
  public Object node () {
    return node;
  }

  /** Returns a String representation containing the current position. 
  */
  public String toString () {
    return st==null ? "unconnected" : st.toString();
  }
}
